
package com.ca.pki.util;

import org.apache.log4j.Logger;

import java.io.*;
import java.awt.*;

/**
 * This class provides methods for retrieving data or image files
 * from a jar file.
 *
 * @bug need to check if the whole data has been completely retrieved.
 *
 * @author vbui
 */
public class GetResource
{
	static Logger logger = Logger.getLogger(GetResource.class.getPackage().getName());

	private static final char fsc = File.separatorChar;
	private static final char jarFsc = '/';

	/**
	 * Default constructor.
	 */
	public GetResource()
	{
	}

	/**
	 * This method fisrt look at the current directory to see if the
	 * file can be found, if not then the jar file will be examined.
	 *
	 * @param dataFileName a String in the form "path\filename.dat".
	 * It is the user's job to make sure "path" does not contain ".\" nor "..\".
	 * @return an InputStream connected to the file or the jar.
	 * @exception FileNotFoundException if file is not in the current dir or in the jar.
	 */
	public InputStream getInputStream(String dataFileName)
			throws FileNotFoundException
	{

		InputStream in = null;

		// try at the current dir first
		try
		{
			in = new FileInputStream(dataFileName);
		}
		catch (FileNotFoundException ex)
		{
			// now try to to read the file from the jar file
			in = getClass().getResourceAsStream(jarFsc + dataFileName.replace(fsc, jarFsc));
		}

		if (in == null)
		{
			throw new FileNotFoundException(dataFileName
					+ " not found in the current dir or in the jar.");
		}

		return in;
	}

	/**
	 * This method fisrt look at the current directory to see if the
	 * file can be found, if not then the jar file will be examined.
	 *
	 * @param imageFileName a String in the form "path\filename.gif".
	 * It is the user's job to make sure "path" does not contain ".\" nor "..\".
	 * @return an Image connected to the file or the jar.
	 * @exception FileNotFoundException if file is not in the current dir or in the jar.
	 */
	public Image getImage(String imageFileName)
			throws FileNotFoundException
	{

		InputStream in = null;

		// try at the current dir first
		try
		{
			in = new FileInputStream(imageFileName);
		}
		catch (FileNotFoundException ex)
		{
			// now try to to read the file from the jar file
			in = getClass().getResourceAsStream(jarFsc + imageFileName.replace(fsc, jarFsc));
		}

		if (in == null)
		{
			throw new FileNotFoundException(imageFileName
					+ " not found in the current dir or in the jar.");
		}

		Toolkit tk = Toolkit.getDefaultToolkit();
		Image img = null;

		try
		{
			byte imageBytes[]=new byte[in.available()];
			in.read(imageBytes);
			img = tk.createImage(imageBytes);
		}
		catch (Exception ex)
		{
			throw new FileNotFoundException(imageFileName
					+ " is not in correct format.");
		}
		finally
		{
			try
			{
				in.close();
			}
			catch (IOException ioe)
			{
				// ignore
			}
			in = null;
		}

		return img;
	}

	/**
	 * This method wraps getImage, but supressed the exception,
	 * simply returning null if the image is not found.  An
	 * error message is printed to System.err.
	 *
	 * @param imageFileName a String in the form "path\filename.gif".
	 * It is the user's job to make sure "path" does not contain ".\" nor "..\".
	 * @return an Image connected to the file or the jar, or null.
	 */
	public Image getImageWithoutException(String imageFileName)
	{
		try
		{
			return getImage(imageFileName);
		}
		catch (Exception e)
		{
			logger.error("unable to get image: " + imageFileName + " -> " + e);
			return null;
		}
	}
}
