
package com.ca.pki.util;

import java.awt.*;
import javax.swing.*;

import java.io.*;
import java.util.*;

import java.text.SimpleDateFormat;

import com.ca.commons.cbutil.CBFileFilter;

import org.apache.log4j.*;
import org.apache.log4j.helpers.NullEnumeration;


/**
 * This class provides utility functions as static methods.
 *
 * @author vbui
 */
public class StaticUtil
{
	static Logger logger = Logger.getLogger(StaticUtil.class.getPackage().getName());
	/**
	 * Return the parent frame of the component c.
	 */
	public static Frame getFrame(Component c)
	{
		if (c == null)
			return null;
		else if(c instanceof Frame)
			return (Frame)c;

		Frame frame = null;

		while((c = c.getParent()) != null)
			if (c == null)
				return null;
			else if(c instanceof Frame)
				frame = (Frame)c;

		return frame;
	}

	/**
	 * Return the parent window of the component c.
	 */
	public static Window getWindow(Component c)
	{
		if (c == null)
			return null;
		else if(c instanceof Window)
			return (Window)c;

		Window window = null;

		while((c = c.getParent()) != null)
			if (c == null)
				return null;
			else if(c instanceof Window)
				window = (Window)c;

		return window;
	}

	/**
	 * Locate component at the middle of the parent frame.
	 *
	 * @param me the component we are setting bounds for.
	 * @param width the preferred width of this component.
	 * @param width the preferred height of this component.
	 * @param parent the parent component.
	 */
	public static void setBounds(Component me, int width, int height,
			Component parent)
	{
		Window parentWindow = getWindow(parent);

		Dimension parentSize = null;
		Point parentLocation = null;

		if (parentWindow == null)
		{
			parentSize = me.getToolkit().getScreenSize();
			parentLocation = new Point(0, 0);
		}
		else
		{
			parentSize = parentWindow.getSize();
			parentLocation = parentWindow.getLocationOnScreen();
		}

		int x = parentLocation.x + (parentSize.width - width) / 2;
		int y = parentLocation.y + (parentSize.height - height) / 2;
		me.setBounds(x, y, width, height);
	}

	public static void setLookAndFeelC(Component comp)
	{
		setLookAndFeel();

		/*
		 *    Setting the font can break the default java
		 *    components (e.g. file browser etc.) on
		 *    non-English localised machines (presumably since they are
		 *    pre translated, and may not have the correct font
		 *    available for the particular unicode characters
		 *    used).  Hence in non-English locales, use the default font.
		 *        - Chris Betts
		 */
		if (Locale.getDefault().getLanguage().equals("en"))
			setDefaultFont(Constants.normFont);

		SwingUtilities.updateComponentTreeUI(comp);
	}

	public static void setLookAndFeel()
	{
		Properties p = System.getProperties();
		String os = p.getProperty("os.name", "Windows");
		try {
			if (os.indexOf("Windows") >= 0)
				UIManager.setLookAndFeel(
					  "com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			else
				UIManager.setLookAndFeel(
							"com.sun.java.swing.plaf.motif.MotifLookAndFeel");
		} catch (Exception exc2) {
			logger.error("Could not load LookAndFeel:\n" + exc2);
		}
	}


	/**
	 * Set default system font.
	 */
	public static void setDefaultFont(Font afont)
	{
		UIManager.put("Button.font", afont);
		UIManager.put("ToggleButton.font", afont);
		UIManager.put("RadioButton.font", afont);
		UIManager.put("CheckBox.font", afont);
		UIManager.put("ColorChooser.font", afont);
		UIManager.put("Label.font", afont);
		UIManager.put("List.font", afont);
		UIManager.put("MenuBar.font", afont);
		UIManager.put("MenuItem.font", afont);
		UIManager.put("MenuItem.acceleratorFont", afont);
		UIManager.put("RadioButtonMenuItem.font", afont);
		UIManager.put("RadioButtonMenuItem.acceleratorFont", afont);
		UIManager.put("CheckBoxMenuItem.font", afont);
		UIManager.put("CheckBoxMenuItem.acceleratorFont", afont);
		UIManager.put("Menu.font", afont);
		UIManager.put("Menu.acceleratorFont", afont);
		UIManager.put("PopupMenu.font", afont);
		UIManager.put("OptionPane.font", afont);
		UIManager.put("Panel.font", afont);
		UIManager.put("ProgressBar.font", afont);
		UIManager.put("ScrollPane.font", afont);
		UIManager.put("Viewport.font", afont);
		UIManager.put("TabbedPane.font", afont);
		UIManager.put("Table.font", afont);
		UIManager.put("TableHeader.font", afont);
		UIManager.put("TitledBorder.font", afont);
		UIManager.put("ToolBar.font", afont);
		UIManager.put("ToolTip.font", afont);
		UIManager.put("Tree.font", afont);
		UIManager.put("ComboBox.font", afont);
		UIManager.put("TextField.font", afont);
		UIManager.put("PasswordField.font", afont);
		UIManager.put("TextArea.font", afont);
		UIManager.put("TextPane.font", afont);
		UIManager.put("EditorPane.font", afont);
		UIManager.put("InternalFrame.font", afont);
		UIManager.put("DestopIcon.font", afont);
	}

	/**
	 *	Checks the value of the property 'Configuration' in %pkihome%\path_properties.config
	 * If the configuration has run successfully, this will be >0; if not pki still needs to be
	 * configured.
	 *@return boolean - whether PKI's has been configured yet
	 */
	public static boolean pkiConfigured()
	{
		int configureRuns=-1;
		//load the properties file
		try
		{
			Properties props = StaticUtil.loadProperties(System.getProperty("PKIHOME") + File.separator +"path_properties.config");
			configureRuns = Integer.parseInt(props.getProperty("Configuration"));
		}
		catch (Exception e)
		{
			System.out.println("exception in StaticUtil.pkiConfigured() ");
			if(logger.isDebugEnabled()){logger.debug(StaticUtil.getStackTraceAsString(e));}
		}
		return (configureRuns > 0);
	}

	/**
	 * Show file chooser to get a file location for saving data.
	 */
	public static String chooseFileToSave(Component parent, String title)
	{
		return chooseFileToSave(parent, title, null, null);
	}

	/**
	 * Show file chooser to get a file location for saving data.
	 */
	public static String chooseFileToSave(Component parent, String title, String[] filter, String fileType)
	{
		JFileChooser chooser = new JFileChooser(System.getProperty("PKIHOME"));
		chooser.setToolTipText(title);
		chooser.setDialogTitle(title);

		if (filter != null && fileType != null)
		{
			CBFileFilter filt = new CBFileFilter(filter, fileType);
			chooser.setFileFilter(filt);
		}

		int returnVal = chooser.showSaveDialog(parent);

		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			if (chooser.getSelectedFile() != null)
				return chooser.getSelectedFile().toString();
		}

		return null;
	}

	/**
	 * Show file chooser to get a file location for data loading.
	 */
	public static String chooseFileToLoad(Component parent, String title)
	{
		return chooseFileToLoad(parent, title, null, null);
	}

	/**
	 * Show file chooser to get a file location for data loading.
	 */
	public static String chooseFileToLoad(Component parent, String title, String[] filter, String fileType)
	{
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		chooser.setToolTipText(title);
		chooser.setDialogTitle(title);

		if (filter != null && fileType != null)
		{
			CBFileFilter filt = new CBFileFilter(filter, fileType);
			chooser.setFileFilter(filt);
		}

		int returnVal = chooser.showOpenDialog(parent);

		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			if (chooser.getSelectedFile() != null)
				return chooser.getSelectedFile().toString();
		}

		return null;
	}

	/**
	 *    This positions a component to the center of another component.
	 *    If both components are showing on the sceen, it uses absolute
	 *    screen co-ordinates, otherwise if only the positioner component
	 *    is showing, it uses relative co-ordinates (since it is unable to
	 *    obtain screen co-ords).  If the components share a reference
	 *    frame, these two actions are equivalent (i.e. if they both have
	 *    the same parent).  If nothing is showing, the component is unchanged.
	 *    @param centreMe the component to center
	 *    @param positioner the component used as the reference center.
	 */
	public static void center(Component centerMe, Component positioner)
	{
		if (positioner.isShowing())
		{
			Rectangle pos = positioner.getBounds();    // relative info.
			Point absPos = positioner.getLocationOnScreen();  // absolute info.
			int centerX = absPos.x + (pos.width/2);    // center x pos, in screen co-ords
			int centerY = absPos.y + (pos.height/2);   // center y pos, in screen co-ords
			pos = centerMe.getBounds();                 // relative info;

			if (centerMe.isShowing())  // if centerMe is showing, center it using screen co-ords (no possibility of error)
			{
				absPos = centerMe.getLocationOnScreen();    // absolute info;
				int currentX = absPos.x + (pos.width/2);    // center of centerMe x pos, in screen co-ords
				int currentY = absPos.y + (pos.height/2);   // center of centerMe y pos, in screen co-ords

				int deltaX = centerX - currentX ;            // amount to move X
				int deltaY = centerY - currentY ;            // amount to move Y
				centerMe.setLocation(pos.x + deltaX, pos.y + deltaY);  // move, using local co-ordinates.
			}
			else  // centerMe isn't showing - can't use screen co-ords, so *assume* both positioner and centerMe have same reference frame
			{     // (i.e. components share a common parent...)
				centerMe.setLocation(centerX - (pos.width/2), centerY - (pos.height/2));
			}
		}
		// else center on screen?
	}

	/**
	 * Loads properties from a file
	 */
	public static Properties loadProperties(String filename)
	{
		Properties check = new Properties();
		FileInputStream in = null;

		try
		{
			in = new FileInputStream(filename);
			check.load(in);
			in.close();
			return check;
		}
		catch (Exception ex)
		{
			System.out.println("Could not open properties file "+filename);
			return null;
		}
	}

	public static void storeProperties(String filename, Properties check, String header)
	{
		FileOutputStream out = null;
		try
		{
			out = new FileOutputStream(filename);
			check.store(out, header);
			out.close();
		}
		catch (Exception ex)
		{
			if(logger.isDebugEnabled()){logger.debug(StaticUtil.getStackTraceAsString(ex));}
			System.out.println("FATAL error: Could not write properties file!");
		}
	}


	/**
	 * Checks for the existence of a directory; if it doesn't already exist
	 * creates it
	 * @param String directoryPath
	 */
	 public static void checkDirectory(String directoryPath)
	 {
System.out.println("checking for existence of " +directoryPath +" directory");
		File directory = new File(directoryPath);
		 if (!directory.exists())
		 {
System.out.println(directoryPath+ " - doesn't exist, about to make it");
			directory.mkdirs();
		 }
	 }

	/**
	 * Converts 'in' to a hexadecimal character string.
	 */
	public static String bytes2Hex(byte[] in) {
		if (in == null) {
			throw new NullPointerException();
		}
		return bytes2Hex(in, 0, in.length);
	}

	/**
	 * Converts 'len' bytes to a hexadecimal character string.
	 */
	public static String bytes2Hex(byte[] in, int offset, int len) {
		if (in == null || offset < 0 || len <= 0
		|| offset + len > in.length) {
			throw new IllegalArgumentException();
		}
		int t;
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < len; i++) {
			t = (in[i+offset] >> 4) & 0x0f;
			if (t < 10) {
				buf.append((char) (t + '0'));
			} else {
				buf.append((char) (t - 10 + 'A'));
			}
			t = in[i+offset] & 0x0f;
			if (t < 10) {
				buf.append((char) (t + '0'));
			} else {
				buf.append((char) (t - 10 + 'A'));
			}
		}
		return new String(buf);
	}

	public static String bytes2HexSplit(byte[] in, int wordlength)
	{
		String hex = bytes2Hex(in);
		StringBuffer buff = new StringBuffer();

		for (int i = 0; i < hex.length(); i++)
		{
			buff.append(hex.charAt(i));
			if ((i+1) % wordlength == 0)
				buff.append(" ");
		}

		return buff.toString();
	}

	public static String bytes2HexSplit(byte[] in, int wordlength, int linelength)
	{
		String hex = bytes2Hex(in);
		StringBuffer buff = new StringBuffer();

		for (int i = 0; i < hex.length(); i++)
		{
			buff.append(hex.charAt(i));
			if ((i+1) % linelength == 0)
			{
				buff.append("\n");
				continue;
			}
			if ((i+1) % wordlength == 0)
				buff.append(" ");
		}

		return buff.toString();
	}

	/**
	 * Converts a hexadecimal character string to a byte array.
	 */
	public static byte [] hex2Bytes(String in) {
		if (in == null || in.length() <= 0) {
			return null;
		}

		if (((in.length() / 2) * 2) != in.length()) {
			in = "0".concat(in);
		}

		int t;
		byte [] out = new byte[in.length()/2];
		for (int i = 0; i < in.length() / 2; i++) {
			t = in.charAt(i * 2);
			if ((t <= '9') && (t >= '0')) {
				out[i] = (byte) ((t - '0') & 0x0f);
			} else if ((t <= 'f') && (t >= 'a')) {
				out[i] = (byte) ((t - 'a' + 10) & 0x0f);
			} else if ((t <= 'F') && (t >= 'A')) {
				out[i] = (byte) ((t - 'A' + 10) & 0x0f);
			} else {
				throw new NumberFormatException();
			}
			out[i] = (byte) ((out[i] << 4) & 0xff);

			t = in.charAt(i * 2 + 1);
			if ((t <= '9') && (t >= '0')) {
				out[i] |= (byte) ((t - '0') & 0x0f);
			} else if ((t <= 'f') && (t >= 'a')) {
				out[i] |= (byte) ((t - 'a' + 10) & 0x0f);
			} else if ((t <= 'F') && (t >= 'A')) {
				out[i] |= (byte) ((t - 'A' + 10) & 0x0f);
			} else {
				throw new NumberFormatException();
			}
		}
		return out;
	}

	/**
	 * check if a drive is ready (windows specific code)
	 * return 0 if success
	 */
	public static int checkDrive(char drive)
	{
		int retval = -1000;
		try
		{
			Process proc = Runtime.getRuntime().exec("cmd /c vol " + drive + ":");
			retval = proc.waitFor();
			proc.destroy();
		}
		catch (Exception ex)
		{
			if(logger.isDebugEnabled()){logger.debug(StaticUtil.getStackTraceAsString(ex));}
		}
		return retval;
	}

	public static boolean okToWriteFile(Frame parent, String fileName)
	{
		// check if the device ready
		if (fileName.indexOf(":") == 1)
		{
			while(checkDrive(fileName.charAt(0)) != 0)
			{
				int retvalue = JOptionPane.showConfirmDialog(parent,
						"The device is not ready. Please check if " + fileName.charAt(0) + ": is properly inserted.",
						"Error", JOptionPane.YES_NO_OPTION);

				if (retvalue != JOptionPane.YES_OPTION)
				{
					System.out.println("Checking aborted");
					return false;
				}
			}
		}

		File f = new File(fileName);
		if (f.isDirectory())
		{
			JOptionPane.showMessageDialog(parent, fileName + " is a directory.", "Error!", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		else if (f.exists())
		{
			int saveAnswer = JOptionPane.showConfirmDialog(parent,
				"File " + fileName + " already exists.\nDo you want to overwrite?",
				"Question", JOptionPane.OK_CANCEL_OPTION);
			if (saveAnswer == JOptionPane.OK_OPTION)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		return true;
	}

	// listing of all the valid revocation reasons (used in crls)
	public static final String[] revocationReasons =
			{"unused", "keyCompromise", "caCompromise", "affiliationChanged",
			 "superseded", "cessationOfOperation", "certificateHold",
			 "removeFromCRL"};

	// Hashtable for easy lookup of revocation code from reason text
	private static Hashtable reasonsLookup = null;

	/**
	 * Returns revocation code from reason text
	 *
	 * @param reason revocation reason in text format, should be one of the revocationReasons
	 * @return 0 to 6 if the reason is valid, or -1 otherwise
	 */
	public static int reasonCode(String reason)
	{
		if (reasonsLookup == null)
		{
			// setup the look up table
			reasonsLookup = new Hashtable();
			for (int i = 0; i < revocationReasons.length; i++)
				reasonsLookup.put(revocationReasons[i], new Integer(i));
		}

		Integer code = (Integer) reasonsLookup.get(reason);
		if (code != null && code.intValue() >= 0 && code.intValue() <= 6)
			return code.intValue();

		return -1;
	}

	/**
	 * Returns revocation text from reason code
	 *
	 * @param code revocation reason code, should be from 0 to 7
	 */
	public static String reasonText(int code)
	{
		if (code >= 0 && code < revocationReasons.length)
			return revocationReasons[code];

		return "unrecognized";
	}

	/**
	 * convert a local time to GMT
	 */
	public static Date toGMT(Date localTime)
	{
		Calendar calendar = Calendar.getInstance();
		int rawOffset = calendar.get(Calendar.ZONE_OFFSET) + calendar.get(Calendar.DST_OFFSET);
		return new Date(localTime.getTime() - rawOffset);
	}

	/**
	 * convert a local time to GMT date string which can be understood by OpenSSLWrap
	 * make it accurate to seconds if the given date is today's date
	 */
	public static String toGMTDateString(String dateString)
	{
		Date date = getDate(dateString);
		if (date == null) date = new Date();

		SimpleDateFormat formatter = null;

		if (isToday(date))
		{
			date = new Date();

			date = toGMT(date);

			formatter = new SimpleDateFormat("ss/mm/HH/MM/dd/yyyy");
		}
		else
			formatter = new SimpleDateFormat("MM/dd/yyyy");

		return formatter.format(date);
	}

	/**
	 * Checks a date if it is today's date by matching yyyy, MM, dd
	 */
	public static boolean isToday(Date date)
	{
		return isSameDay(date, new Date());
	}

	/**
	 * Checks if two (yyyy,MM,dd) dates are the same
	 */
	public static boolean isSameDay(Date date1, Date date2)
	{
		Calendar now = Calendar.getInstance();
		now.setTime(date1);

		Calendar then = Calendar.getInstance();
		then.setTime(date2);

		if (now.get(Calendar.YEAR) == then.get(Calendar.YEAR) &&
				now.get(Calendar.MONTH) == then.get(Calendar.MONTH) &&
				now.get(Calendar.DAY_OF_MONTH) == then.get(Calendar.DAY_OF_MONTH))
		{
			return true;
		}

		return false;
	}

	/**
	 * Parses a date field, accept both "MM/dd/yyyy" or "now +/- years" format
	 */
	public static Date getDate(String value)
	{
		SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
		Calendar cal = Calendar.getInstance();

		// validating
		try
		{
			// format "now (+/- ys)"
			if (value.startsWith("now"))
			{
				int signInd = 0;
				if ((signInd = value.indexOf("+")) > 0)
				{
					int years = Integer.parseInt(value.substring(signInd + 1).trim());
					cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) + years);
				}
				else if ((signInd = value.indexOf("-")) > 0)
				{
					int years = Integer.parseInt(value.substring(signInd + 1).trim());
					cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) - years);
				}

				return cal.getTime();
			}
			// format "MM/dd/yyyy"
			else
			{
				return formatter.parse(checkDate(value));
			}
		}
		catch (Exception ex)
		{
			if ( ! (ex.getMessage()).substring(0,16).equals("Unparseable date"))
			{
				System.out.print(ex.getMessage());
			}
			return null;
		}
	}

	/**
	 * Check if a date is in correct MM/dd/yyyy format
	 *
	 * @throw Exception with the detailed error message
	 */
	public static String checkDate(String dateString) throws Exception
	{
		// more sensible error message for blank date
		if (dateString == null || dateString.trim().length() == 0)
			throw new Exception("The date field cannot be left blank!");

		// check year, ensure the year token has 4 digits, if it has only
		// two digits xx, then make it 20xx
		StringTokenizer tok = new StringTokenizer(dateString, "/");
		if (tok.countTokens() < 3)
			throw new Exception("The date string must contains 3 tokens MM/dd/yyyy");

		String MM = tok.nextToken().trim();
		String dd = tok.nextToken().trim();
		String yyyy = tok.nextToken().trim();

		//check that MM contains only numbers
		if ( MM.length() != 2 )
		{
			throw new Exception("A valid month must be two characters long\nEg: '02' for February");
		}
		if ( ! Character.isDigit(MM.charAt(0)) )
		{
			throw new Exception("A month must be a numeric value.\nIt cannot contain '" + MM.charAt(0) + "'.");
		}
		if (! Character.isDigit(MM.charAt(1)) )
		{
			throw new Exception("A month must be a numeric value.\nIt cannot contain '" + MM.charAt(1) + "'.");
		}
		int MMValue = Integer.parseInt(MM);
		// validate MM
		if (MMValue > 12 || MMValue < 1)
			throw new Exception("Valid month value must be from 1 to 12");

		//check that dd contains only numbers
		if ( dd.length() != 2)
		{
			throw new Exception("A valid day must be two characters long.\nEg: '03' for the third");
		}
		if (! Character.isDigit(dd.charAt(0)) )
		{
			throw new Exception("A day must be a numeric value. It cannot\n contain '" + dd.charAt(0) + "'.");
		}
		if ( ! Character.isDigit(dd.charAt(1)) )
		{
			throw new Exception("A day must be a numeric value. It cannot\n contain '" + dd.charAt(1) + "'.");
		}
		// validate dd
		int ddValue = Integer.parseInt(dd);
		if (ddValue > 31 || ddValue < 1)
			throw new Exception("Valid day value must be from 1 to 31");

		// validate yyyy,
		if (yyyy.length() != 4)
		{
			throw new Exception("The year token must be 4 characters long.");
		}
		else if (yyyy.length() == 4)
		{
			for (int i=0; i<4; i++)
			{
				if (! Character.isDigit(yyyy.charAt(i)) )
				{
					throw new Exception("The year must be a numeric value. \nIt cannot contain '" + yyyy.charAt(i) + "'.");
				}
			}
			int yyVal = Integer.parseInt(yyyy);
			GregorianCalendar currentDate = new GregorianCalendar();
			if (yyVal < currentDate.get(GregorianCalendar.YEAR))
				throw new Exception("The year must be " + currentDate.get(GregorianCalendar.YEAR)+ " or later.");
		}
		return MM + "/" + dd + "/" + yyyy;
	}

	/**
	 * Parses a "MM/dd/yyyy" or "now +/- years" date field, returns a date string in format "MM/dd/yyyy"
	 */
	public static String parseDate(String value)
	{
		SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
		Date d = getDate(value);

		if (d == null)
			return formatter.format(new Date());
		else
			return formatter.format(d);
	}

	/**
	 * Replace all the appreances of oldPattern in a source string by newPattern.
	 */
	public static String replace(String source, String oldPattern, String newPattern)
	{
		String temp = source;
		int found = -1;
		while ((found = temp.indexOf(oldPattern)) >= 0)
		{
			temp = temp.substring(0, found) + newPattern + temp.substring(found + oldPattern.length());
		}
		return temp;
	}

	public static String getGeneralizedTime(Date adate)
	{
		adate = StaticUtil.toGMT(adate);

		SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
		return formatter.format(adate) + "Z";
	}


	/**
	 * Write a log item and also pop an error dialog
	 */
	public static void logAndDialog(Logger logger, Priority logLevel, String message, Frame parentFrame, int dialogType)
	{
		logger.log(logLevel, message);

		JOptionPane.showMessageDialog(parentFrame,
				message, "Information", dialogType);
	}


	public static String getFullHostName(String host) {
		try {
			Process p = Runtime.getRuntime().exec("nslookup "+host);
			InputStream is = p.getInputStream();
			int ret = p.waitFor();
			int len = is.available();
			byte [] buf = new byte [len];
			is.read(buf);
			is.close();
			String lookup = new String(buf);
			int dnsInd = lookup.indexOf("Name:");
			if ( dnsInd > 0 ) {
				int addInd = lookup.indexOf("Addr", dnsInd);
				if ( addInd > 0 ) {
					String dns = lookup.substring(dnsInd+5, addInd-1).trim();
					return dns;
				}
			}
		} catch (Exception e) {
			if(logger.isDebugEnabled()){logger.debug(StaticUtil.getStackTraceAsString(e));}
		}
		return host;
	}

	public static String getStackTraceAsString(Exception e)
	{
		//StringWriter to contain the contents of the StackTrace
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		//output the stackTrace to stdout
		//if(logger.isDebugEnabled()){logger.debug(StaticUtil.getStackTraceAsString(e));}
		//get the stackTrace and fill the printwriter
		e.printStackTrace(pw);
		//stringbuffer will hold the contents of the stackTrace
		StringBuffer error = sw.getBuffer();
		//close the printStream
		pw.close();

		return error.toString();
	}
}